@@ -3,7 +3,7 @@
 //
 // Contents: Routines that use connection handles
 //
-// Microsoft Drivers 5.3 for PHP for SQL Server
+// Microsoft Drivers 5.6 for PHP for SQL Server
 // Copyright(c) Microsoft Corporation
 // All rights reserved.
 // MIT License
@@ -17,7 +17,11 @@
 //  IN THE SOFTWARE.
 //---------------------------------------------------------------------------------------------------------------------------------
 
-#include "php_sqlsrv.h"
+extern "C" {
+  #include "php_sqlsrv.h"
+}
+
+#include "php_sqlsrv_int.h"
 
 #include <string>
 #include <sstream>
@@ -46,6 +50,45 @@ struct date_as_string_func {
     }
 };
 
+struct format_decimals_func
+{
+    static void func(connection_option const* /*option*/, _In_ zval* value, _Inout_ sqlsrv_conn* conn, std::string& /*conn_str*/ TSRMLS_DC)
+    {
+        TSRMLS_C;   // show as used to avoid a warning
+
+        ss_sqlsrv_conn* ss_conn = static_cast<ss_sqlsrv_conn*>(conn);
+
+        if (zend_is_true(value)) {
+            ss_conn->format_decimals = true;
+        }
+        else {
+            ss_conn->format_decimals = false;
+        }
+    }
+};
+
+struct decimal_places_func
+{
+
+    static void func(connection_option const* /*option*/, _In_ zval* value, _Inout_ sqlsrv_conn* conn, std::string& /*conn_str*/ TSRMLS_DC)
+    {
+        TSRMLS_C;   // show as used to avoid a warning
+
+        // first check if the input is an integer
+        if (Z_TYPE_P(value) != IS_LONG) {
+            THROW_SS_ERROR(conn, SQLSRV_ERROR_INVALID_DECIMAL_PLACES);
+        }
+
+        zend_long decimal_places = Z_LVAL_P(value);
+        if (decimal_places < 0 || decimal_places > SQL_SERVER_MAX_MONEY_SCALE) {
+            decimal_places = NO_CHANGE_DECIMAL_PLACES;
+        }
+
+        ss_sqlsrv_conn* ss_conn = static_cast<ss_sqlsrv_conn*>(conn);
+        ss_conn->decimal_places = static_cast<short>(decimal_places);
+    }
+};
+
 struct conn_char_set_func {
 
     static void func( connection_option const* /*option*/, _Inout_ zval* value, _Inout_ sqlsrv_conn* conn, std::string& /*conn_str*/ TSRMLS_DC )
@@ -173,6 +216,9 @@ namespace SSStmtOptionNames {
     const char SEND_STREAMS_AT_EXEC[] = "SendStreamParamsAtExec";
     const char SCROLLABLE[] = "Scrollable";
     const char CLIENT_BUFFER_MAX_SIZE[] = INI_BUFFERED_QUERY_LIMIT;
+    const char DATE_AS_STRING[] = "ReturnDatesAsStrings";
+    const char FORMAT_DECIMALS[] = "FormatDecimals";
+    const char DECIMAL_PLACES[] = "DecimalPlaces";
 }
 
 namespace SSConnOptionNames {
@@ -180,6 +226,7 @@ namespace SSConnOptionNames {
 // most of these strings are the same for both the sqlsrv_connect connection option
 // and the name put into the connection string. MARS is the only one that's different.
 const char APP[] = "APP";
+const char AccessToken[] = "AccessToken";
 const char ApplicationIntent[] = "ApplicationIntent";
 const char AttachDBFileName[] = "AttachDbFileName";
 const char Authentication[] = "Authentication";
@@ -189,6 +236,8 @@ const char ConnectionPooling[] = "ConnectionPooling";
 const char ConnectRetryCount[] = "ConnectRetryCount";
 const char ConnectRetryInterval[] = "ConnectRetryInterval";
 const char Database[] = "Database";
+const char DecimalPlaces[] = "DecimalPlaces";
+const char FormatDecimals[] = "FormatDecimals";
 const char DateAsString[] = "ReturnDatesAsStrings";
 const char Driver[] = "Driver";
 const char Encrypt[] = "Encrypt";
@@ -214,6 +263,8 @@ const char WSID[] = "WSID";
 enum SS_CONN_OPTIONS {
     
     SS_CONN_OPTION_DATE_AS_STRING = SQLSRV_CONN_OPTION_DRIVER_SPECIFIC,
+    SS_CONN_OPTION_FORMAT_DECIMALS,
+    SS_CONN_OPTION_DECIMAL_PLACES,
 };
 
 //List of all statement options supported by this driver
@@ -242,6 +293,24 @@ const stmt_option SS_STMT_OPTS[] = {
         SQLSRV_STMT_OPTION_CLIENT_BUFFER_MAX_SIZE, 
         std::unique_ptr<stmt_option_buffered_query_limit>( new stmt_option_buffered_query_limit )
     },
+    {
+        SSStmtOptionNames::DATE_AS_STRING, 
+        sizeof( SSStmtOptionNames::DATE_AS_STRING ),
+        SQLSRV_STMT_OPTION_DATE_AS_STRING, 
+        std::unique_ptr<stmt_option_date_as_string>( new stmt_option_date_as_string )
+    },
+    {
+        SSStmtOptionNames::FORMAT_DECIMALS, 
+        sizeof( SSStmtOptionNames::FORMAT_DECIMALS ),
+        SQLSRV_STMT_OPTION_FORMAT_DECIMALS, 
+        std::unique_ptr<stmt_option_format_decimals>( new stmt_option_format_decimals )
+    },
+    {
+        SSStmtOptionNames::DECIMAL_PLACES, 
+        sizeof( SSStmtOptionNames::DECIMAL_PLACES),
+        SQLSRV_STMT_OPTION_DECIMAL_PLACES, 
+        std::unique_ptr<stmt_option_decimal_places>( new stmt_option_decimal_places )
+    },
     { NULL, 0, SQLSRV_STMT_OPTION_INVALID, std::unique_ptr<stmt_option_functor>{} },
 };
 
@@ -257,6 +326,15 @@ const connection_option SS_CONN_OPTS[] = {
         CONN_ATTR_STRING,
         conn_str_append_func::func 
     },
+    {
+        SSConnOptionNames::AccessToken,
+        sizeof( SSConnOptionNames::AccessToken ),
+        SQLSRV_CONN_OPTION_ACCESS_TOKEN,
+        ODBCConnOptions::AccessToken,
+        sizeof( ODBCConnOptions::AccessToken), 
+        CONN_ATTR_STRING,
+        access_token_set_func::func
+    },
     { 
         SSConnOptionNames::ApplicationIntent,
         sizeof( SSConnOptionNames::ApplicationIntent ),
@@ -491,6 +569,24 @@ const connection_option SS_CONN_OPTS[] = {
         CONN_ATTR_BOOL,
         date_as_string_func::func
     },
+    {
+        SSConnOptionNames::FormatDecimals,
+        sizeof( SSConnOptionNames::FormatDecimals),
+        SS_CONN_OPTION_FORMAT_DECIMALS,
+        SSConnOptionNames::FormatDecimals,
+        sizeof( SSConnOptionNames::FormatDecimals),
+        CONN_ATTR_BOOL,
+        format_decimals_func::func
+    },
+    {
+        SSConnOptionNames::DecimalPlaces,
+        sizeof( SSConnOptionNames::DecimalPlaces),
+        SS_CONN_OPTION_DECIMAL_PLACES,
+        SSConnOptionNames::DecimalPlaces,
+        sizeof( SSConnOptionNames::DecimalPlaces),
+        CONN_ATTR_INT,
+        decimal_places_func::func
+    },
     { NULL, 0, SQLSRV_CONN_OPTION_INVALID, NULL, 0 , CONN_ATTR_INVALID, NULL },  //terminate the table
 };
 
@@ -978,7 +1074,7 @@ PHP_FUNCTION( sqlsrv_prepare )
 
             // Initialize the options array to be passed to the core layer
             ALLOC_HASHTABLE( ss_stmt_options_ht );
-            core::sqlsrv_zend_hash_init( *conn , ss_stmt_options_ht, 3 /* # of buckets */, 
+            core::sqlsrv_zend_hash_init( *conn , ss_stmt_options_ht, 5 /* # of buckets */, 
                                          ZVAL_PTR_DTOR, 0 /*persistent*/ TSRMLS_CC );
             
             validate_stmt_options( *conn, options_z, ss_stmt_options_ht TSRMLS_CC );
@@ -1101,7 +1197,7 @@ PHP_FUNCTION( sqlsrv_query )
 
             // Initialize the options array to be passed to the core layer
             ALLOC_HASHTABLE( ss_stmt_options_ht );
-            core::sqlsrv_zend_hash_init( *conn , ss_stmt_options_ht, 3 /* # of buckets */, ZVAL_PTR_DTOR, 
+            core::sqlsrv_zend_hash_init( *conn , ss_stmt_options_ht, 5 /* # of buckets */, ZVAL_PTR_DTOR, 
                                          0 /*persistent*/ TSRMLS_CC );
             
             validate_stmt_options( *conn, options_z, ss_stmt_options_ht TSRMLS_CC );    
@@ -1234,12 +1330,12 @@ void sqlsrv_conn_close_stmts( _Inout_ ss_sqlsrv_conn* conn TSRMLS_DC )
 
 int get_conn_option_key( _Inout_ sqlsrv_context& ctx, _In_ zend_string* key, _In_ size_t key_len, _Inout_ zval const* value_z TSRMLS_DC )    
 {
-    for( int i=0; SS_CONN_OPTS[ i ].conn_option_key != SQLSRV_CONN_OPTION_INVALID; ++i )
+    for( int i=0; SS_CONN_OPTS[i].conn_option_key != SQLSRV_CONN_OPTION_INVALID; ++i )
     {
-        if( key_len == SS_CONN_OPTS[ i ].sqlsrv_len && !stricmp( ZSTR_VAL( key ), SS_CONN_OPTS[ i ].sqlsrv_name )) {
+        if( key_len == SS_CONN_OPTS[i].sqlsrv_len && !stricmp( ZSTR_VAL( key ), SS_CONN_OPTS[i].sqlsrv_name )) {
             
 
-             switch( SS_CONN_OPTS[ i ].value_type ) {
+             switch( SS_CONN_OPTS[i].value_type ) {
 
                 case CONN_ATTR_BOOL:                        
                     // bool attributes can be either strings to be appended to the connection string
@@ -1250,7 +1346,7 @@ int get_conn_option_key( _Inout_ sqlsrv_context& ctx, _In_ zend_string* key, _In
                 case CONN_ATTR_INT:
                 {
                     CHECK_CUSTOM_ERROR( (Z_TYPE_P( value_z ) != IS_LONG ), ctx, SQLSRV_ERROR_INVALID_OPTION_TYPE_INT,
-                                        SS_CONN_OPTS[ i ].sqlsrv_name ) 
+                                        SS_CONN_OPTS[i].sqlsrv_name ) 
                     {
                         throw ss::SSException();
                     }
@@ -1259,7 +1355,7 @@ int get_conn_option_key( _Inout_ sqlsrv_context& ctx, _In_ zend_string* key, _In
                 case CONN_ATTR_STRING:
                 {
                     CHECK_CUSTOM_ERROR( Z_TYPE_P( value_z ) != IS_STRING, ctx, SQLSRV_ERROR_INVALID_OPTION_TYPE_STRING, 
-                                        SS_CONN_OPTS[ i ].sqlsrv_name ) {
+                                        SS_CONN_OPTS[i].sqlsrv_name ) {
 
                         throw ss::SSException();
                     }
@@ -1268,7 +1364,7 @@ int get_conn_option_key( _Inout_ sqlsrv_context& ctx, _In_ zend_string* key, _In
                     size_t value_len = Z_STRLEN_P( value_z );
                     bool escaped = core_is_conn_opt_value_escaped( value, value_len );
 
-                    CHECK_CUSTOM_ERROR( !escaped, ctx, SS_SQLSRV_ERROR_CONNECT_BRACES_NOT_ESCAPED, SS_CONN_OPTS[ i ].sqlsrv_name ) {
+                    CHECK_CUSTOM_ERROR( !escaped, ctx, SS_SQLSRV_ERROR_CONNECT_BRACES_NOT_ESCAPED, SS_CONN_OPTS[i].sqlsrv_name ) {
 
                         throw ss::SSException();
                     }
@@ -1278,7 +1374,7 @@ int get_conn_option_key( _Inout_ sqlsrv_context& ctx, _In_ zend_string* key, _In
                         valid = core_is_authentication_option_valid( value, value_len );
                     }
 
-                    CHECK_CUSTOM_ERROR( !valid, ctx, SS_SQLSRV_ERROR_INVALID_AUTHENTICATION_OPTION, SS_CONN_OPTS[ i ].sqlsrv_name ) {
+                    CHECK_CUSTOM_ERROR( !valid, ctx, SS_SQLSRV_ERROR_INVALID_AUTHENTICATION_OPTION, SS_CONN_OPTS[i].sqlsrv_name ) {
 
                         throw ss::SSException();
                     }
@@ -1287,7 +1383,7 @@ int get_conn_option_key( _Inout_ sqlsrv_context& ctx, _In_ zend_string* key, _In
                 }
             }
 
-            return SS_CONN_OPTS[ i ].conn_option_key;
+            return SS_CONN_OPTS[i].conn_option_key;
         }
     }
     return SQLSRV_CONN_OPTION_INVALID;
@@ -1295,10 +1391,10 @@ int get_conn_option_key( _Inout_ sqlsrv_context& ctx, _In_ zend_string* key, _In
 
 int get_stmt_option_key( _In_ zend_string* key, _In_ size_t key_len TSRMLS_DC )
 {
-    for( int i = 0; SS_STMT_OPTS[ i ].key != SQLSRV_STMT_OPTION_INVALID; ++i )
+    for( int i = 0; SS_STMT_OPTS[i].key != SQLSRV_STMT_OPTION_INVALID; ++i )
     {
-        if( key_len == SS_STMT_OPTS[ i ].name_len && !stricmp( ZSTR_VAL( key ), SS_STMT_OPTS[ i ].name )) {
-            return SS_STMT_OPTS[ i ].key;
+        if( key_len == SS_STMT_OPTS[i].name_len && !stricmp( ZSTR_VAL( key ), SS_STMT_OPTS[i].name )) {
+            return SS_STMT_OPTS[i].key;
         }
     }
     return SQLSRV_STMT_OPTION_INVALID;
