@@ -3,7 +3,7 @@
 //
 // Contents: common initialization routines shared by PDO and sqlsrv
 //
-// Microsoft Drivers 5.3 for PHP for SQL Server
+// Microsoft Drivers 5.6 for PHP for SQL Server
 // Copyright(c) Microsoft Corporation
 // All rights reserved.
 // MIT License
@@ -159,20 +159,3 @@ void core_sqlsrv_mshutdown( _Inout_ sqlsrv_context& henv_cp, _Inout_ sqlsrv_cont
     return;
 }
 
-
-// DllMain for the extension.  
-#ifdef _WIN32
-BOOL WINAPI DllMain( _In_ HINSTANCE hinstDLL, _In_ DWORD fdwReason, LPVOID )
-{
-    switch( fdwReason ) {
-        case DLL_PROCESS_ATTACH:
-            // store the module handle for use by client_info and server_info
-            g_sqlsrv_hmodule = hinstDLL;
-            break;
-        default:
-            break;
-    }
-
-    return TRUE;
-}
-#endif
