@@ -5,7 +5,7 @@
 // 
 // Comments: Mostly error handling and some type handling
 //
-// Microsoft Drivers 5.3 for PHP for SQL Server
+// Microsoft Drivers 5.6 for PHP for SQL Server
 // Copyright(c) Microsoft Corporation
 // All rights reserved.
 // MIT License
@@ -28,7 +28,7 @@ log_callback g_driver_log;
 // internal error that says that FormatMessage failed
 SQLCHAR INTERNAL_FORMAT_ERROR[] = "An internal error occurred.  FormatMessage failed writing an error message.";
 // buffer used to hold a formatted log message prior to actually logging it.
-char last_err_msg[ 2048 ];  // 2k to hold the error messages
+char last_err_msg[2048] = {'\0'};  // 2k to hold the error messages
 
 // routine used by utf16_string_from_mbcs_string
 unsigned int convert_string_from_default_encoding( _In_ unsigned int php_encoding, _In_reads_bytes_(mbcs_len) char const* mbcs_in_string,
@@ -91,25 +91,6 @@ bool convert_string_from_utf16_inplace( _In_ SQLSRV_ENCODING encoding, _Inout_up
     return result;
 }
 
-bool convert_zval_string_from_utf16( _In_ SQLSRV_ENCODING encoding, _Inout_ zval* value_z, _Inout_ SQLLEN& len)
-{
-    char* string = Z_STRVAL_P(value_z);
-
-    if( validate_string(string, len)) {
-       return true;
-    }
-
-    char* outString = NULL;
-    SQLLEN outLen = 0;
-    bool result = convert_string_from_utf16( encoding, reinterpret_cast<const SQLWCHAR*>(string), int(len / sizeof(SQLWCHAR)), &outString, outLen );
-    if( result ) {
-       core::sqlsrv_zval_stringl( value_z, outString, outLen );
-       sqlsrv_free( outString );
-       len = outLen;
-    }
-    return result;
-}
-
 bool validate_string( _In_ char* string, _In_ SQLLEN& len )
 {
      SQLSRV_ASSERT(string != NULL, "String must be specified");
@@ -146,10 +127,13 @@ bool convert_string_from_utf16( _In_ SQLSRV_ENCODING encoding, _In_reads_bytes_(
         flags = WC_ERR_INVALID_CHARS;
     }
 
-    // calculate the number of characters needed
 #ifndef _WIN32
-    cchOutLen = SystemLocale::FromUtf16Strict( encoding, inString, cchInLen, NULL, 0 );
+    // Allocate enough space to hold the largest possible number of bytes for UTF-8 conversion
+    // instead of calling FromUtf16, for performance reasons
+    cchOutLen = 4*cchInLen;
 #else	
+    // Calculate the number of output bytes required - no performance hit here because
+    // WideCharToMultiByte is highly optimised
     cchOutLen = WideCharToMultiByte( encoding, flags,
                                    inString, cchInLen, 
                                    NULL, 0, NULL, NULL );
@@ -161,9 +145,10 @@ bool convert_string_from_utf16( _In_ SQLSRV_ENCODING encoding, _In_reads_bytes_(
 
     // Create a buffer to fit the encoded string
     char* newString = reinterpret_cast<char*>( sqlsrv_malloc( cchOutLen + 1 /* NULL char*/ ));
+    memset(newString, '\0', cchOutLen+1);
     
 #ifndef _WIN32
-    int rc = SystemLocale::FromUtf16( encoding, inString, cchInLen, newString, static_cast<int>(cchOutLen));
+    int rc = SystemLocale::FromUtf16Strict( encoding, inString, cchInLen, newString, static_cast<int>(cchOutLen));
 #else
     int rc = WideCharToMultiByte( encoding, flags, inString, cchInLen, newString, static_cast<int>(cchOutLen), NULL, NULL );
 #endif // !_WIN32
@@ -172,9 +157,13 @@ bool convert_string_from_utf16( _In_ SQLSRV_ENCODING encoding, _In_reads_bytes_(
         sqlsrv_free( newString );
         return false;
     }
+    char* newString2 = reinterpret_cast<char*>( sqlsrv_malloc( rc + 1 /* NULL char*/ ));
+    memset(newString2, '\0', rc+1);
+    memcpy_s(newString2, rc, newString, rc);
+    sqlsrv_free( newString );
 
-    *outString = newString;
-    newString[cchOutLen] = '\0';   // null terminate the encoded string
+    *outString = newString2;
+    cchOutLen = rc;
 
     return true;
 }
@@ -219,8 +208,8 @@ bool core_sqlsrv_get_odbc_error( _Inout_ sqlsrv_context& ctx, _In_ int record_nu
 
     SQLRETURN r = SQL_SUCCESS;
     SQLSMALLINT wmessage_len = 0;
-    SQLWCHAR wsqlstate[ SQL_SQLSTATE_BUFSIZE ] = { L'\0' };
-    SQLWCHAR wnative_message[ SQL_MAX_ERROR_MESSAGE_LENGTH + 1 ] = { L'\0' };
+    SQLWCHAR wsqlstate[SQL_SQLSTATE_BUFSIZE] = {L'\0'};
+    SQLWCHAR wnative_message[SQL_MAX_ERROR_MESSAGE_LENGTH + 1] = {L'\0'};
     SQLSRV_ENCODING enc = ctx.encoding();
 
     switch( h_type ) {
@@ -265,10 +254,36 @@ bool core_sqlsrv_get_odbc_error( _Inout_ sqlsrv_context& ctx, _In_ int record_nu
             // We need to calculate number of characters
             SQLINTEGER wsqlstate_len = sizeof( wsqlstate ) / sizeof( SQLWCHAR );
             SQLLEN sqlstate_len = 0;
-            convert_string_from_utf16(enc, wsqlstate, wsqlstate_len, (char**)&error->sqlstate, sqlstate_len);
 
+            convert_string_from_utf16(enc, wsqlstate, wsqlstate_len, (char**)&error->sqlstate, sqlstate_len);
+            
             SQLLEN message_len = 0;
-            convert_string_from_utf16(enc, wnative_message, wmessage_len, (char**)&error->native_message, message_len);
+            if (r == SQL_SUCCESS_WITH_INFO && wmessage_len > SQL_MAX_ERROR_MESSAGE_LENGTH) {
+                // note that wmessage_len is the number of characters required for the error message -- 
+                // create a new buffer big enough for this lengthy error message
+                sqlsrv_malloc_auto_ptr<SQLWCHAR> wnative_message_str;
+
+                SQLSMALLINT expected_len = wmessage_len * sizeof(SQLWCHAR);
+                SQLSMALLINT returned_len = 0;
+
+                wnative_message_str = reinterpret_cast<SQLWCHAR*>(sqlsrv_malloc(expected_len));
+                memset(wnative_message_str, '\0', expected_len); 
+
+                SQLRETURN rtemp = ::SQLGetDiagFieldW(h_type, h, record_number, SQL_DIAG_MESSAGE_TEXT, wnative_message_str, wmessage_len, &returned_len);
+                if (!SQL_SUCCEEDED(rtemp) || returned_len != expected_len) {
+                    // something went wrong
+                    return false;
+                }
+
+                convert_string_from_utf16(enc, wnative_message_str, wmessage_len, (char**)&error->native_message, message_len);
+            } else {
+                convert_string_from_utf16(enc, wnative_message, wmessage_len, (char**)&error->native_message, message_len);
+            }
+
+            if (message_len == 0 && error->native_message == NULL) {
+                // something went wrong
+                return false;
+            }
             break;
     }
 
@@ -352,11 +367,11 @@ void die( _In_opt_ const char* msg, ... )
     va_start( format_args, msg );
     DWORD rc = FormatMessage( FORMAT_MESSAGE_FROM_STRING, msg, 0, 0, last_err_msg, sizeof( last_err_msg ), &format_args );
     va_end( format_args );
-    if( rc == 0 ) {
-        php_error( E_ERROR, reinterpret_cast<const char*>( INTERNAL_FORMAT_ERROR ));
+    if (rc == 0) {
+        php_error(E_ERROR, "%s", reinterpret_cast<const char*>(INTERNAL_FORMAT_ERROR));
     }
 
-    php_error( E_ERROR, last_err_msg );
+    php_error(E_ERROR, "%s", last_err_msg);
 }
 
 namespace {
@@ -393,7 +408,7 @@ unsigned int convert_string_from_default_encoding( _In_ unsigned int php_encodin
     if( required_len == 0 ) {
         return 0;
     }
-    utf16_out_string[ required_len ] = '\0';
+    utf16_out_string[required_len] = '\0';
 
     return required_len;
 }
