@@ -3,7 +3,7 @@
 //
 // Contents: Routines that use statement handles
 //
-// Microsoft Drivers 5.3 for PHP for SQL Server
+// Microsoft Drivers 5.6 for PHP for SQL Server
 // Copyright(c) Microsoft Corporation
 // All rights reserved.
 // MIT License
@@ -18,7 +18,12 @@
 //---------------------------------------------------------------------------------------------------------------------------------
 
 // *** header files ***
-#include "php_sqlsrv.h"
+extern "C" {
+  #include "php_sqlsrv.h"
+}
+
+#include "php_sqlsrv_int.h"
+
 #ifdef _WIN32
 #include <sal.h>
 #endif // _WIN32
@@ -91,7 +96,7 @@ const char SS_SQLSRV_WARNING_PARAM_VAR_NOT_REF[] = "Variable parameter %d not pa
 /* internal functions */
 
 void convert_to_zval( _Inout_ sqlsrv_stmt* stmt, _In_ SQLSRV_PHPTYPE sqlsrv_php_type, _In_opt_ void* in_val, _In_ SQLLEN field_len, _Inout_ zval& out_zval );
-
+SQLSMALLINT get_resultset_meta_data(_Inout_ sqlsrv_stmt* stmt);
 void fetch_fields_common( _Inout_ ss_sqlsrv_stmt* stmt, _In_ zend_long fetch_type, _Out_ zval& fields, _In_ bool allow_empty_field_names
 						TSRMLS_DC );
 bool determine_column_size_or_precision( sqlsrv_stmt const* stmt, _In_ sqlsrv_sqltype sqlsrv_type, _Inout_ SQLULEN* column_size,
@@ -110,6 +115,15 @@ bool verify_and_set_encoding( _In_ const char* encoding_string, _Inout_ sqlsrv_p
 
 }
 
+// internal helper function to free meta data structures allocated
+void meta_data_free( _Inout_ field_meta_data* meta )
+{
+    if( meta->field_name ) {
+        meta->field_name.reset();
+    }
+    sqlsrv_free( meta );
+}
+
 // query options for cursor types
 namespace SSCursorTypes {
 
@@ -129,15 +143,24 @@ ss_sqlsrv_stmt::ss_sqlsrv_stmt( _In_ sqlsrv_conn* c, _In_ SQLHANDLE handle, _In_
     fetch_fields_count ( 0 )
 {
     core_sqlsrv_set_buffered_query_limit( this, SQLSRV_G( buffered_query_limit ) TSRMLS_CC );
+
+    // inherit other values based on the corresponding connection options
+    ss_sqlsrv_conn* ss_conn = static_cast<ss_sqlsrv_conn*>(conn);
+    date_as_string = ss_conn->date_as_string;
+    format_decimals = ss_conn->format_decimals;
+    decimal_places = ss_conn->decimal_places;
 }
 
 ss_sqlsrv_stmt::~ss_sqlsrv_stmt( void )
 {
+    std::for_each(current_meta_data.begin(), current_meta_data.end(), meta_data_free);
+    current_meta_data.clear();
+
     if( fetch_field_names != NULL ) {
 
         for( int i=0; i < fetch_fields_count; ++i ) {
             
-            sqlsrv_free( fetch_field_names[ i ].name );
+            sqlsrv_free( fetch_field_names[i].name );
         }
         sqlsrv_free( fetch_field_names );
     }
@@ -155,7 +178,7 @@ void ss_sqlsrv_stmt::new_result_set( TSRMLS_D )
 
         for( int i=0; i < fetch_fields_count; ++i ) {
             
-            sqlsrv_free( fetch_field_names[ i ].name );
+            sqlsrv_free( fetch_field_names[i].name );
         }
         sqlsrv_free( fetch_field_names );
     }
@@ -230,7 +253,7 @@ sqlsrv_phptype ss_sqlsrv_stmt::sql_type_to_php_type( _In_ SQLINTEGER sql_type, _
         case SQL_SS_TIMESTAMPOFFSET:
         case SQL_SS_TIME2:
         case SQL_TYPE_TIMESTAMP:
-            if( reinterpret_cast<ss_sqlsrv_conn*>( this->conn )->date_as_string ) { 
+            if (this->date_as_string) {
                 ss_phptype.typeinfo.type = SQLSRV_PHPTYPE_STRING;
                 ss_phptype.typeinfo.encoding = this->conn->encoding();
             }
@@ -455,27 +478,24 @@ PHP_FUNCTION( sqlsrv_field_metadata )
 
     try {
 
-    // get the number of fields in the resultset
-    num_cols = core::SQLNumResultCols( stmt TSRMLS_CC );
+    // get the number of fields in the resultset and its metadata if not exists
+    SQLSMALLINT num_cols = get_resultset_meta_data(stmt);
 
     zval result_meta_data;
     ZVAL_UNDEF( &result_meta_data );
     core::sqlsrv_array_init( *stmt, &result_meta_data TSRMLS_CC );
     
     for( SQLSMALLINT f = 0; f < num_cols; ++f ) {
-    
-        sqlsrv_malloc_auto_ptr<field_meta_data> core_meta_data;
-        core_meta_data = core_sqlsrv_field_metadata( stmt, f TSRMLS_CC );
-
+        field_meta_data* core_meta_data = stmt->current_meta_data[f];
+        
         // initialize the array
         zval field_array;
         ZVAL_UNDEF( &field_array );
         core::sqlsrv_array_init( *stmt, &field_array TSRMLS_CC );
 
-        core::sqlsrv_add_assoc_string( *stmt, &field_array, FieldMetaData::NAME, 
-                                       reinterpret_cast<char*>( core_meta_data->field_name.get() ), 0 TSRMLS_CC );
-
-        core_meta_data->field_name.transferred();
+        // add the field name to the associative array but keep a copy
+        core::sqlsrv_add_assoc_string(*stmt, &field_array, FieldMetaData::NAME,
+                                      reinterpret_cast<char*>(core_meta_data->field_name.get()), 1 TSRMLS_CC);
 
         core::sqlsrv_add_assoc_long( *stmt, &field_array, FieldMetaData::TYPE, core_meta_data->field_type TSRMLS_CC );
 
@@ -515,9 +535,6 @@ PHP_FUNCTION( sqlsrv_field_metadata )
        
         // add this field's meta data to the result set meta data
         core::sqlsrv_add_next_index_zval( *stmt, &result_meta_data, &field_array TSRMLS_CC );
-
-        // always good to call destructor for allocations done through placement new operator.
-        core_meta_data->~field_meta_data();
     }
 
     // return our built collection and transfer ownership
@@ -563,6 +580,10 @@ PHP_FUNCTION( sqlsrv_next_result )
 
         core_sqlsrv_next_result( stmt TSRMLS_CC, true );
 
+        // clear the current meta data since the new result will generate new meta data
+        std::for_each(stmt->current_meta_data.begin(), stmt->current_meta_data.end(), meta_data_free);
+        stmt->current_meta_data.clear();
+
         if( stmt->past_next_result_end ) {
 
             RETURN_NULL();
@@ -719,7 +740,7 @@ PHP_FUNCTION( sqlsrv_num_fields )
     }
 }
 
-// sqlsrv_fetch_object( resource $stmt [, string $className [, array $ctorParams ]])
+// sqlsrv_fetch_object( resource $stmt [, string $className [, array $ctorParams]])
 // 
 // Retrieves the next row of data as a PHP object.
 //
@@ -889,7 +910,9 @@ PHP_FUNCTION( sqlsrv_fetch_object )
             fci.object = Z_OBJ_P( &retval_z );
 
             memset( &fcic, 0, sizeof( fcic ));
+#if PHP_VERSION_ID < 70300
             fcic.initialized = 1;
+#endif
             fcic.function_handler = class_entry->constructor;
             fcic.calling_scope = class_entry;
 
@@ -1078,7 +1101,7 @@ PHP_FUNCTION( sqlsrv_get_field )
     try {
 
         // validate that the field index is within range
-        int num_cols = core::SQLNumResultCols( stmt TSRMLS_CC );
+        SQLSMALLINT num_cols = get_resultset_meta_data(stmt);
 
         if( field_index < 0 || field_index >= num_cols ) {
             THROW_SS_ERROR( stmt, SS_SQLSRV_ERROR_INVALID_FUNCTION_PARAMETER, _FN_ );
@@ -1616,10 +1639,13 @@ sqlsrv_phptype determine_sqlsrv_php_type( _In_ ss_sqlsrv_stmt const* stmt, _In_
 
     switch( sql_type ) {
         case SQL_BIGINT:
-        case SQL_CHAR:
         case SQL_DECIMAL:
-        case SQL_GUID:
         case SQL_NUMERIC:
+            sqlsrv_phptype.typeinfo.type = SQLSRV_PHPTYPE_STRING;
+            sqlsrv_phptype.typeinfo.encoding = SQLSRV_ENCODING_CHAR;
+            break;
+        case SQL_CHAR:
+        case SQL_GUID:
         case SQL_WCHAR:
             sqlsrv_phptype.typeinfo.type = SQLSRV_PHPTYPE_STRING;
             sqlsrv_phptype.typeinfo.encoding = stmt->encoding();
@@ -1641,6 +1667,7 @@ sqlsrv_phptype determine_sqlsrv_php_type( _In_ ss_sqlsrv_stmt const* stmt, _In_
         case SQL_SMALLINT:
         case SQL_TINYINT:
             sqlsrv_phptype.typeinfo.type = SQLSRV_PHPTYPE_INT;
+            sqlsrv_phptype.typeinfo.encoding = SQLSRV_ENCODING_CHAR;
             break;
         case SQL_BINARY:
         case SQL_LONGVARBINARY:
@@ -1670,14 +1697,14 @@ sqlsrv_phptype determine_sqlsrv_php_type( _In_ ss_sqlsrv_stmt const* stmt, _In_
         case SQL_FLOAT:
         case SQL_REAL:
             sqlsrv_phptype.typeinfo.type = SQLSRV_PHPTYPE_FLOAT;
+            sqlsrv_phptype.typeinfo.encoding = SQLSRV_ENCODING_CHAR;
             break;
         case SQL_TYPE_DATE:
         case SQL_SS_TIMESTAMPOFFSET:
         case SQL_SS_TIME2:
         case SQL_TYPE_TIMESTAMP:
         {
-            ss_sqlsrv_conn* c = static_cast<ss_sqlsrv_conn*>( stmt->conn );
-            if( c->date_as_string ) {
+            if (stmt->date_as_string) { 
                 sqlsrv_phptype.typeinfo.type = SQLSRV_PHPTYPE_STRING;
                 sqlsrv_phptype.typeinfo.encoding = stmt->encoding();
             }
@@ -1754,6 +1781,37 @@ void determine_stmt_has_rows( _Inout_ ss_sqlsrv_stmt* stmt TSRMLS_DC )
     }
 }
 
+SQLSMALLINT get_resultset_meta_data(_Inout_ sqlsrv_stmt * stmt)
+{
+    // get the numer of columns in the result set
+    SQLSMALLINT num_cols = -1;
+    
+    num_cols = stmt->current_meta_data.size();
+    bool getMetaData = false;
+
+    if (num_cols == 0) {
+        getMetaData = true;
+        num_cols = core::SQLNumResultCols(stmt TSRMLS_CC);
+    }
+
+    try {
+        if (getMetaData) {
+            for (int i = 0; i < num_cols; i++) {
+                sqlsrv_malloc_auto_ptr<field_meta_data> core_meta_data;
+                core_meta_data = core_sqlsrv_field_metadata(stmt, i TSRMLS_CC);
+                stmt->current_meta_data.push_back(core_meta_data.get());
+                core_meta_data.transferred();
+            }
+        }
+    } catch( core::CoreException& ) {
+        throw;
+    }
+
+    SQLSRV_ASSERT(stmt->current_meta_data.size() == num_cols, "Meta data vector out of sync" );
+
+    return num_cols;
+}
+
 void fetch_fields_common( _Inout_ ss_sqlsrv_stmt* stmt, _In_ zend_long fetch_type, _Out_ zval& fields, _In_ bool allow_empty_field_names
 						TSRMLS_DC )
 {
@@ -1767,49 +1825,38 @@ void fetch_fields_common( _Inout_ ss_sqlsrv_stmt* stmt, _In_ zend_long fetch_typ
 		throw ss::SSException();
 	}
 
-	// get the numer of columns in the result set
-	SQLSMALLINT num_cols = core::SQLNumResultCols(stmt TSRMLS_CC);
+    // get the numer of columns in the result set and its metadata if not exists
+    SQLSMALLINT num_cols = get_resultset_meta_data(stmt);
 
 	// if this is the first fetch in a new result set, then get the field names and
 	// store them off for successive fetches.
-	if(( fetch_type & SQLSRV_FETCH_ASSOC ) && stmt->fetch_field_names == NULL ) {
+    if ((fetch_type & SQLSRV_FETCH_ASSOC) && stmt->fetch_field_names == NULL) {
 
         SQLLEN field_name_len = 0;
-        SQLSMALLINT field_name_len_w = 0;
-        SQLWCHAR field_name_w[( SS_MAXCOLNAMELEN + 1 ) * 2 ] = { L'\0' };
-        sqlsrv_malloc_auto_ptr<char> field_name;
         sqlsrv_malloc_auto_ptr<sqlsrv_fetch_field_name> field_names;
-        field_names = static_cast<sqlsrv_fetch_field_name*>( sqlsrv_malloc( num_cols * sizeof( sqlsrv_fetch_field_name )));
-        SQLSRV_ENCODING encoding = (( stmt->encoding() == SQLSRV_ENCODING_DEFAULT ) ? stmt->conn->encoding() : stmt->encoding());
-        for( int i = 0; i < num_cols; ++i ) {
-
-            core::SQLColAttributeW ( stmt, i + 1, SQL_DESC_NAME, field_name_w, ( SS_MAXCOLNAMELEN + 1 ) * 2, &field_name_len_w, NULL TSRMLS_CC );
-
-            //Conversion function expects size in characters
-            field_name_len_w = field_name_len_w / sizeof ( SQLWCHAR );
-            bool converted = convert_string_from_utf16( encoding, field_name_w,
-                field_name_len_w, ( char** ) &field_name, field_name_len );
-
-            CHECK_CUSTOM_ERROR( !converted, stmt, SQLSRV_ERROR_FIELD_ENCODING_TRANSLATE, get_last_error_message() ) {
-                throw core::CoreException();
-            }
-
-            field_names[i].name = static_cast<char*>( sqlsrv_malloc( field_name_len, sizeof( char ), 1 ));
-            memcpy_s(( void* )field_names[i].name, ( field_name_len * sizeof( char )) , ( void* ) field_name, field_name_len );
-            field_names[i].name[field_name_len] = '\0';  // null terminate the field name since SQLColAttribute doesn't.
-            field_names[i].len = field_name_len + 1;
-            field_name.reset();
+        field_names = static_cast<sqlsrv_fetch_field_name*>(sqlsrv_malloc(num_cols * sizeof(sqlsrv_fetch_field_name)));
+        for (int i = 0; i < num_cols; ++i) {
+            // The meta data field name is already null-terminated, and the field name len is correct.
+            field_name_len = stmt->current_meta_data[i]->field_name_len;
+            field_names[i].name = static_cast<char*>(sqlsrv_malloc(field_name_len, sizeof(char), 1));
+            memcpy_s((void*)field_names[i].name, (field_name_len * sizeof(char)), (void*)stmt->current_meta_data[i]->field_name, field_name_len);
+            field_names[i].name[field_name_len] = '\0';     // null terminate the field name after the memcpy
+            field_names[i].len = field_name_len;            // field_name_len should not need to include the null char
         }
-		
+
         stmt->fetch_field_names = field_names;
         stmt->fetch_fields_count = num_cols;
         field_names.transferred();
     }
 
-    int zr = array_init( &fields );
-	CHECK_ZEND_ERROR( zr, stmt, SQLSRV_ERROR_ZEND_HASH ) {
-		throw ss::SSException();
-	}
+    int zr = SUCCESS;
+#if PHP_VERSION_ID < 70300
+    CHECK_ZEND_ERROR(array_init(&fields), stmt, SQLSRV_ERROR_ZEND_HASH) {
+        throw ss::SSException();
+    }
+#else
+    array_init(&fields); 
+#endif 
 
 	for( int i = 0; i < num_cols; ++i ) {
 		SQLLEN field_len = -1;
@@ -1831,12 +1878,12 @@ void fetch_fields_common( _Inout_ ss_sqlsrv_stmt* stmt, _In_ zend_long fetch_typ
 
 		if( fetch_type & SQLSRV_FETCH_ASSOC ) {
 
-			CHECK_CUSTOM_WARNING_AS_ERROR(( stmt->fetch_field_names[i].len == 1 && !allow_empty_field_names ), stmt,
+			CHECK_CUSTOM_WARNING_AS_ERROR(( stmt->fetch_field_names[i].len == 0 && !allow_empty_field_names ), stmt,
 											SS_SQLSRV_WARNING_FIELD_NAME_EMPTY) {
 				throw ss::SSException();
 			}
 
-			if( stmt->fetch_field_names[ i ].len > 1 || allow_empty_field_names ) {
+			if( stmt->fetch_field_names[i].len > 0 || allow_empty_field_names ) {
 
 				zr = add_assoc_zval( &fields, stmt->fetch_field_names[i].name, &field );
 				CHECK_ZEND_ERROR( zr, stmt, SQLSRV_ERROR_ZEND_HASH ) {
